/*
 *  GameApp.h
 *  
 *
 *  Created by Mark Szymczyk 
 *  Copyright (c) 2006 Me and Mark Publishing. All rights reserved.
 *
 */

#include <Carbon/Carbon.h>
#include <AGL/agl.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>

#ifndef GAME_TEXTURE
    #include "GameTexture.h"
#endif

#ifndef GAME_LEVEL
    #include "GameLevel.h"
#endif

#ifndef GAME_TILE
    #include "GameTile.h"
#endif

#ifndef GAME_CONTEXT
    #include "GameContext.h"
#endif

#define GAME_APP


class GameApp
{
	protected:
        EventLoopTimerUPP gameTimer;
        EventLoopTimerRef timer;
        
        GameContextPtr context;
        GameLevelPtr currentLevel;
		GameTileListPtr currentTileList;

        
	public:
		// Constructor and destructor
        GameApp(void);
		virtual ~GameApp(void);
        
        // Accessors
        GameContextPtr GetContext(void);
        void SetContext(GameContextPtr theContext);
        
        GameLevelPtr GetCurrentLevel(void);
        void SetCurrentLevel(GameLevelPtr theLevel);
        
		void InitApp(void);
		void EventLoop(void);
		void CleanUpApp(void);

		// Initialization functions
		void InitToolbox (void);
		void SetUpMenuBar(void);
        void RegisterMenuCommands(void);
        void InstallKeyboardEventHandler(void);
        void InstallTimer(void);
        void LoadLevel(CFStringRef filename, CFStringRef extension, CFStringRef subdirectory, short levelID);
        void LoadTileList(CFStringRef filename, CFStringRef extension, CFStringRef subdirectory, short levelID);
        void InitializeDrawContext(short width, short height);

		// menu related functions
		static pascal OSStatus MenuEventHandler(EventHandlerCallRef myHandlerChain,
                EventRef event, void* userData);
		virtual void Quit(void);


		// Game related functions
		static pascal void GameLoopTimer(EventLoopTimerRef theTimer, void* userData);
        static pascal OSStatus KeyboardEventHandler(EventHandlerCallRef myHandlerChain,
                                                EventRef event, void* userData);
		void GameLoop(void);
		void RenderFrame(void);
		void Pause(void);
		void Resume(void);

};

typedef GameApp* GameAppPtr;
typedef GameApp** GameAppHandle;


// constants for the menus and menu items

const short kMenuBarID = 128;

const short kAppleMenu = 128;
const short kAbout		= 1;

const short kFileMenu = 129;
// element 2 is a line separator
const short kResumeMenuItem = 	1;
const short kFileMenuDivider = 2;
const short	kQuitMenuItem = 	3;		
const short kResumeTextID = 128;

// Keyboard constants
const short kEscapeKey = 0x35;
const short kQuitKey = 0x0C00;
const short kLeftArrow = 0x7B;
const short kRightArrow = 0x7C;
const short kDownArrow = 0x7D;
const short kUpArrow = 0x7E;


const char kLetterQ = 'Q';

const Boolean kUseVirtualKeyCode = true;
const Boolean kDontUseVirtualKeyCode = false;

const MenuCommand kCommandResumeGame = FOUR_CHAR_CODE('Rsum');
const short kEventTotal = 1;
const short kKeyboardEventTotal = 1;

//const short k16BitColor = 16;
//const short k32BitColor = 32;

const short kSingleBuffered = 1;
const short kDoubleBuffered = 2;

const short kOneTile = 1;