/*
 *  GameContext.h
 *  MGPCode(OpenGL)
 *
 *  Created by Mark Szymczyk on 4/18/06.
 *  Copyright 2006 Me and Mark Publishing. All rights reserved.
 *
 */

#include <Carbon/Carbon.h>
#include <AGL/agl.h>

#ifndef GAME_TILE
    #include "GameTile.h"
#endif

#ifndef GAME_LEVEL
    #include "GameLevel.h"
#endif

#ifndef GAME_TEXTURE
    #include "GameTexture.h"
#endif

#define GAME_CONTEXT

class GameContext 
{
    protected:
        AGLContext aglDrawContext;
        GameTexturePtr tileStorage;
        
        // The offsets are used for scrolling. They make sure
        // the proper area of the game world is drawn on the screen.
        short xOffset;
        short yOffset;
        
        short widthInTiles;
        short heightInTiles;
        
    public:
        // Constructor and destructor
        GameContext(void);
        virtual ~GameContext(void);
        
        // Accessors
        AGLContext GetAGLDrawContext(void);
        void SetAGLDrawContext(AGLContext theContext);
        
        GameTexturePtr GetTileStorage(void);
        void SetTileStorage(GameTexturePtr theStorage);
        
        short GetXOffset(void);
        void SetXOffset(short offset);
        
        short GetYOffset(void);
        void SetYOffset(short offset);
        
        short GetWidthInTiles(void);
        void SetWidthInTiles(short width);
        
        short GetHeightInTiles(void);
        void SetHeightInTiles(short height);
        
        // AGL functions
        void InitializeAGL(void);
        void TurnOnFullscreen(void);
        void TurnOffFullscreen(void);
        
        // Scrolling functions
		short ScrollLeft(GameLevelPtr theLevel, short tilesToScroll);
		short ScrollRight(GameLevelPtr theLevel, short tilesToScroll);
		short ScrollUp(GameLevelPtr theLevel, short tilesToScroll);
		short ScrollDown(GameLevelPtr theLevel, short tilesToScroll);
        
        // Drawing functions
        void DrawTiles(GameLevelPtr theLevel);
        void DrawTileFromLevelMap(GameLevelPtr theLevel, short theRow, short theColumn);

        // Utility functions
		Point GetGridLocation(short tileNumber);
};

typedef GameContext* GameContextPtr;
typedef GameContext** GameContextHandle;

// Constants
const short kTilesPerTextureRow = 32;
const short kTilesPerTextureColumn = 32;
        