// Level.h  	
// Written by Mark Szymczyk

#ifndef __CARBON__
    #include <Carbon/Carbon.h>
#endif

#define GAME_LEVEL


class MapElement
{
	protected:
		short value;	// The tile number
		Boolean visibleToPlayer;
		
	public:	
		short GetValue(void);  			
		void SetValue(short theValue);	
		
		Boolean IsVisibleToPlayer(void) ;				
		void SetVisibleToPlayer(Boolean visibility);	
};


typedef MapElement* MapElementPtr;
typedef MapElement** MapElementHandle;

class GameLevel
{
	public:
		// Normally you do not want to make data members public, but
		// using accessor functions to retrieve arrays is a pain.  Its
		// easier to just make the array public and read it directly.
		MapElementPtr levelMap;	// Used as a 2D array
        
	protected:
		short levelWidth;		// in tiles
		short levelHeight;		// in tiles
		
        // The code from the book has been modified to read levels and tile
        // lists from external files in the application bundle. So there is no
        // longer any need to store the tile ID. If you have several tile sets
        // in your game, you'll want to add a data member that identifies the tile
        // set this level uses.
        //short tileID;			// Resource ID of the tiles this level uses
		
	public:
		// constructor
		GameLevel(void);
		GameLevel(const GameLevel& theLevel);	// Copy constructor
		
		// Destructor
		~GameLevel(void);
		
		// Overload assignment operator
		GameLevel& operator= (const GameLevel& theLevel);

		void AllocateLevelMap(void);
		void DeleteLevelMap(void);
		void InitializeLevelMap (short value);
				
		// Accessor functions
		short GetLevelWidth(void);
		void SetLevelWidth(short width);
		short GetLevelHeight(void);
		void SetLevelHeight(short height);
		
		//short GetTileID(void);
		//void SetTileID(short theID);
	
		// File related functions
        Boolean Load(CFStringRef filename, CFStringRef extension, CFStringRef subdirectory, short levelID);	
        Boolean ReadLevelData(FSRef* filename, short levelID);		
		
};

typedef GameLevel* GameLevelPtr;
typedef GameLevel** GameLevelHandle;

// non-member functions	
Boolean SeriousError(OSErr errorCode);

// constants
const short kEmpty = -1;
const short kNoForegroundTile = -1;

const short k2Bytes = 2;
const short k4Bytes = 4;

const short kCurrentLevelVersion = 2;
const short kLevelResourceID = 201;
const OSType kLevelFileType = 'Levl';
const ResType kLevelResourceType = 'Lmap';
const Str255 kLevelName = "\pLevel data to save";
