// GameTile.cp		
// Created by Mark Szymczyk 


#ifndef GAME_TILE
	#include "GameTile.h"
#endif

short GameTile::GetTileNumber(void)
{
	return tileNumber;
}
		
void GameTile::SetTileNumber(short theNumber)
{
	tileNumber = theNumber;
}

GameTileType GameTile::GetTileType(void)
{
	return tileType;
}


void GameTile::SetTileType(GameTileType theType)
{
	tileType = theType;
}

// Tile table allocate and delete functions
void GameTileList::AllocateTileTable(short desiredTableSize)
{
	size_t tileTableSize = desiredTableSize * sizeof(GameTile);
	tileTable = (GameTilePtr)NewPtr(tileTableSize);
}

void GameTileList::DeleteTileTable(void)
{
	if (tileTable != nil) {
		DisposePtr((Ptr)tileTable);
		tileTable = nil;
	}
}

// File-related functions
Boolean GameTileList::Load(CFStringRef filename, CFStringRef extension, CFStringRef subdirectory, short tileID)
{
    // You can pass the entire name of the file as the filename argument or you can
    // separate the entire name and its file extension into the
    // filename and fileExtension arguments. If you pass the entire name as the
    // filename argument, pass NULL as the fileExtension argument.
    
    // You can normally pass NULL as the subdirectory. If you place all your
    // texture files in a special directory in your application, you can pass
    // the directory name as the subdirectory argument to make searching faster.
    
    CFBundleRef mainBundle;
    CFURLRef tileURL;
    FSRef tileFile;
    
    mainBundle = CFBundleGetMainBundle();
    tileURL = CFBundleCopyResourceURL(mainBundle, filename, extension, subdirectory);
    
    if (CFURLGetFSRef(tileURL, &tileFile)) {
        return ReadTilesFromDisk(&tileFile, tileID);
    }
    else {
        return false;
    }
}


Boolean GameTileList::ReadTilesFromDisk(FSRef* filename, short tileID)
{
    // This routine has been modified so that it works on both
    // Intel and PowerPC Macs. I created the tiles for this level
    // back in 2001, which makes things easier because I know the
    // data is big endian. After reading every piece of data, I call the
    // functions CFSwapInt16BigToHost and CFSwapInt32BigToHost. 
    // These routines will do nothing on
    // PowerPC Macs and swap the bytes on Intel Macs.
    
	
    short error;
	short refNum;
	Handle savedTileTypeList;
	
    // Open the resource file.
    refNum = FSOpenResFile(filename, fsRdPerm);
    UseResFile(refNum);

    // Read the tiles resource.
	savedTileTypeList = Get1Resource(kTileResourceType, tileID);
	error = ResError();
	if ((error != noErr) || (savedTileTypeList == nil))
		return false;

	short version;
    short numberOfTiles;
	Size offset = 0;
    
	// Read version used to save the tiles.
	BlockMoveData(*savedTileTypeList + offset, &version, k2Bytes);
	offset = offset + k2Bytes;
    version = CFSwapInt16BigToHost(version);

	// If resource is obsolete, don't load it.
	if (version != kCurrentTileVersion) {
		ReleaseResource((Handle)savedTileTypeList);
		//CloseResFile(refNum);
		return false;
	}
	
	// Read the number of tiles in the tile list and allocate enough
    // memory to the tile table to hold all the tiles.
    BlockMoveData(*savedTileTypeList + offset, &numberOfTiles, k2Bytes);
	offset = offset + k2Bytes;
    numberOfTiles = CFSwapInt16BigToHost(numberOfTiles);
	AllocateTileTable(numberOfTiles);
    
	// Now let's fill the tileTypeList array
    short currentTile;
    short currentTileNumber;
    UInt32 currentTileType;
    
    for (currentTile = 0; currentTile < numberOfTiles; currentTile++) {
        // Read the tile number.
        BlockMoveData(*savedTileTypeList + offset, &currentTileNumber, k2Bytes);
        offset = offset + k2Bytes;
        currentTileNumber = CFSwapInt16BigToHost(currentTileNumber);

        // Each tile number is two bytes long, with two bytes of padding.
        // Skip the padding. This is covering for a flaw in the level
        // editor I wrote 5 years ago to make the level. I wrote out the GameTile
        // data structure instead of writing 2 bytes for each tile number
        // and 2 bytes for each tile type.
        offset = offset + k2Bytes;
        
        // Read the tile type.
        BlockMoveData(*savedTileTypeList + offset, &currentTileType, k4Bytes);
        offset = offset + k4Bytes;
        currentTileType = CFSwapInt32BigToHost(currentTileType);
        
        // Add the tile information to the tile table.
        tileTable[currentTile].SetTileNumber(currentTileNumber);
        tileTable[currentTile].SetTileType((GameTileType)currentTileType);
        
    }

	// Clean up
	ReleaseResource(Handle(savedTileTypeList));
	error = ResError();
	if (SeriousError(error))
		return false;
	
	return true;
}

// Utility functions
short GameTileList::GetTileNumber(Point location)
{
	short rowNumber = location.v;
	short columnNumber = location.h;
	
	short result = (rowNumber * kTilesInRow) + columnNumber;
	
	return result;
}

Point GameTileList::GetGridLocation(short tileNumber)
{
	// Given a tile number from the level map, return
	// the grid location of the tile on the tile board.
	Point result;
	
	result.h = tileNumber % kTilesInRow;
	result.v = tileNumber / kTilesInRow;
	
	return result;
}

